package de.opalpiraten.scenepainter.gui;
import java.awt.Color;
import java.awt.Font;
import java.awt.Insets;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.GridLayout;
import java.io.*;    
import javax.swing.*;
import java.awt.event.*;  
import javax.swing.event.*;
import java.beans.*;
import java.net.*;

import de.opalpiraten.scenepainter.sys.*;
import de.opalpiraten.scenepainter.gui.render.*;

/**
 * This Editor is the GUI frontend for the TextSceneElement class
 */
public class TextSceneElementEditor extends JPanel implements 
ActionListener, ChangeListener, SceneListener, KeyListener {

  /**
   * The text to render
   */
  private JTextArea txt = new JTextArea();
  
  /**
   * Name of the Font to use
   */
  private JTextField fnt = new JTextField("default");
  
  /**
   * Fontsize
   */
  private JSpinner fntsize = new JSpinner(new SpinnerNumberModel(10,0,200,1));
  
  /**
   * Color of the text
   */
  private JTextField fntcol = new JTextField("#000000");
  
  /**
   * Color of the outline
   */
  private JTextField outline = new JTextField("#000000");
  
  /**
   * Color of the background
   */
  private JTextField back = new JTextField("#FFFFFF");
  
  /**
   * Fire rendering
   */
  private JButton render = new JButton(Util.createIcon("toolbarButtonGraphics/general/Edit16.gif",""));
  
  /**
   * Adjust the length of the tail
   */
  private JSpinner tail = new JSpinner(new SpinnerNumberModel(25,0,2000,5));
  
  /**
   * Adjust tail position
   */
  private JSpinner tailpos = new JSpinner(new SpinnerNumberModel(0,0,7,1));
  
  /**
   * The styles available
   */
  private String[] styles = {
    TextSceneElement.SPEECH,
    //TextSceneElement.THOUGHT,
    TextSceneElement.EMOTION,
    TextSceneElement.BOX,
    TextSceneElement.OUTLINE,
    TextSceneElement.PLAIN
    
  };
  
  /**
   * Style selected by user
   */
  private JComboBox style = new JComboBox(styles);
  
  /**
   * Reference to the scene showing
   */
  private Scene scene;
  
  /**
   * Reference to the embedding container
   */
  private SceneElementEditor parent;
  
  /**
   * Make sure, setScene() does not become reentrant
   */
  private boolean noupdate=false;
  
  public TextSceneElementEditor(SceneElementEditor see) {
    parent =see;
    
    render.addActionListener(this);
    fntsize.addChangeListener(this);
    fnt.addActionListener(this);
    outline.addActionListener(this);
    back.addActionListener(this);
    style.addActionListener(this);
    fntcol.addActionListener(this);
    tail.addChangeListener(this);
    tailpos.addChangeListener(this);
    txt.addKeyListener(this);
    
    
    GridBagLayout gbl = new GridBagLayout();
    setLayout(gbl);
    GridBagConstraints gbc = new GridBagConstraints();
    
    gbc.anchor=GridBagConstraints.NORTHWEST;
    gbc.fill=GridBagConstraints.BOTH;
    gbc.gridx=0;
    gbc.gridy=0;
    gbc.gridwidth=1; 
    gbc.gridheight=1;
    gbc.weightx=1.0;
    gbc.weighty=1.0;
    gbc.insets = new Insets(1,1,1,1);
    JScrollPane jsp = new JScrollPane(txt);
    gbl.setConstraints(jsp,gbc);
    jsp.setBorder(BorderFactory.createTitledBorder(Dict.get("gui.textsceneelementeditor.txt")));
    add(jsp);
    
    gbc.anchor=GridBagConstraints.NORTHWEST;
    gbc.fill=GridBagConstraints.BOTH;
    gbc.gridx=0;
    gbc.gridy=1;
    gbc.gridwidth=1; 
    gbc.gridheight=1;
    gbc.weightx=0.0;
    gbc.weighty=0.0;
    gbc.insets = new Insets(1,1,1,1);
    JPanel fntpanel = new JPanel();
    fntpanel.setLayout(new BoxLayout(fntpanel,BoxLayout.X_AXIS));
    fntpanel.add(fnt);
    fntpanel.add(fntsize);
    gbl.setConstraints(fntpanel,gbc);
    fntpanel.setBorder(BorderFactory.createTitledBorder(Dict.get("gui.textsceneelementeditor.font")));
    add(fntpanel);
    
    gbc.anchor=GridBagConstraints.NORTHWEST;
    gbc.fill=GridBagConstraints.BOTH;
    gbc.gridx=0;
    gbc.gridy=2;
    gbc.gridwidth=1; 
    gbc.gridheight=1;
    gbc.weightx=0.0;
    gbc.weighty=0.0;
    gbc.insets = new Insets(1,1,1,1);
    
    JPanel colpanel = new JPanel();
    colpanel.setLayout(new GridLayout(3,0,0,3));
    colpanel.add(new JLabel(Dict.get("gui.textsceneelementeditor.color.text")));
    colpanel.add(fntcol);
    colpanel.add(new JLabel(Dict.get("gui.textsceneelementeditor.color.back")));
    colpanel.add(back);
    colpanel.add(new JLabel(Dict.get("gui.textsceneelementeditor.color.outline")));
    colpanel.add(outline);
    gbl.setConstraints(colpanel,gbc);
    colpanel.setBorder(BorderFactory.createTitledBorder(Dict.get("gui.textsceneelementeditor.color")));
    add(colpanel);

    gbc.anchor=GridBagConstraints.NORTHWEST;
    gbc.fill=GridBagConstraints.BOTH;
    gbc.gridx=0;
    gbc.gridy=3;
    gbc.gridwidth=1; 
    gbc.gridheight=1;
    gbc.weightx=0.0;
    gbc.weighty=0.0;
    gbc.insets = new Insets(1,1,1,1);
    JPanel stylepanel = new JPanel();
    stylepanel.setLayout(new GridLayout(3,0,0,3));
    stylepanel.add(new JLabel(Dict.get("gui.textsceneelementeditor.style.type")));
    stylepanel.add(style);
    stylepanel.add(new JLabel(Dict.get("gui.textsceneelementeditor.style.tail")));
    stylepanel.add(tail);
    stylepanel.add(new JLabel(Dict.get("gui.textsceneelementeditor.style.tailpos")));
    stylepanel.add(tailpos);
    gbl.setConstraints(stylepanel,gbc);
    stylepanel.setBorder(BorderFactory.createTitledBorder(Dict.get("gui.textsceneelementeditor.style")));
    add(stylepanel);

    gbc.anchor=GridBagConstraints.SOUTHEAST;
    gbc.fill=GridBagConstraints.NONE;
    gbc.gridx=0;
    gbc.gridy=4;
    gbc.gridwidth=1; 
    gbc.gridheight=1;
    gbc.weightx=0.0;
    gbc.weighty=0.0;
    gbc.insets = new Insets(1,1,1,1);
    gbl.setConstraints(render,gbc);
    add(render);
  }
  
  public void actionPerformed(ActionEvent e) {
    render();
  }
  
  public void keyTyped(KeyEvent e) {}

  public void keyPressed(KeyEvent e) {}

  public void keyReleased(KeyEvent e) {
    if ((e.isAltDown() || e.isControlDown()) && e.getKeyCode()==KeyEvent.VK_ENTER) {
      render();
    }
  }
  
  private void render () {
    if (noupdate) return;
    if (txt.getText().length()==0) return;
    try {
      TextSceneElement tse = new TextSceneElement();
      tse.setText(txt.getText());
      int s = ((Integer)fntsize.getValue()).intValue();
      String fntinfo[] = fnt.getText().split(" *, *");
      int fntstyle=Font.PLAIN;
      for (int i=1;i<fntinfo.length;i++) {
        if (fntinfo[i].toUpperCase().equals("ITALIC")) fntstyle=fntstyle | Font.ITALIC;
        if (fntinfo[i].toUpperCase().equals("BOLD")) fntstyle=fntstyle | Font.BOLD;
      }
      
      tse.setFont(new Font(fntinfo[0],fntstyle,s));
      tse.setTextcolor(Color.decode(fntcol.getText()));
      tse.setBackcolor(Color.decode(back.getText()));
      tse.setBordercolor(Color.decode(outline.getText()));
      tse.setStyle(style.getSelectedItem().toString());
      tse.setTaillength(((Integer)tail.getValue()).intValue());
      tse.setTaildock(((Integer)tailpos.getValue()).intValue());

      if (parent.isCreating() || scene.getSelectedIndex()<0) {
        scene.add(tse);
        scene.setSelectedIndex(0);
      }
      else {
        SceneElement se = scene.getSelected();
        tse.setPosition(se.getPosition());
        tse.setAlpha(se.getAlpha());
        scene.replace(scene.getSelectedIndex(),tse);
      }
    }
    catch (Exception exp) {
      // Lazy about sanity checking
    }
  }
  
  public void replaceScene(Scene s) {
    if (scene!=null) scene.removeSceneListener(this);
    scene=s;
    scene.addSceneListener(this);
    sceneChanged(s);
  }
  
  public void sceneChanged (Scene s) {
    noupdate =true; // Safeguard, keeping this from getting reentrant
    try {
      TextSceneElement tse = (TextSceneElement)s.getSelected();
      txt.setText(tse.getText());
      Font f = tse.getFont();
      String fntinfo=f.getName();
      if (f.isItalic()) fntinfo =fntinfo+", ITALIC";
      if (f.isBold()) fntinfo =fntinfo+", BOLD";
      
      fnt.setText(fntinfo);
      fntsize.setValue(new Integer(f.getSize()));
      tail.setValue(new Integer(tse.getTaillength()));
      tailpos.setValue(new Integer(tse.getTaildock()));
      fntcol.setText(Util.toHexString(tse.getTextcolor()));
      back.setText(Util.toHexString(tse.getBackcolor()));
      outline.setText(Util.toHexString(tse.getBordercolor()));
      style.setSelectedItem(tse.getStyle());
    }
    catch (Exception e) {
      // Be lazy about sanity chicking
    }
    noupdate=false;
  }
  
  public void setProject(Project p) {
  }
  
  public void stateChanged(ChangeEvent e) {
    render();
  }
  
}
