package de.opalpiraten.scenepainter.sys;
import java.util.*;
import java.text.*;
import java.io.*;
import java.net.*;

/**
 * This class is responsible for i18n
 */
public class Dict  {

  /**
   * The "classname" of the resource containing the translations
   */
  public static final String RSRCNAME = "resources.i18n";
  
  /**
   * The "classname" of the resource containint the translations for tools
   */
  public static final String TOOLRSRCNAME = "resources.i18n-tool";
  
  /**
   * Properties loaded frrom tools
   */
  private static Hashtable trans = new Hashtable();
  
  
  /**
   * Just for the sake of having a constructor. This class does not need
   * to be instantiated.
   */
  private Dict() {}
  
  /**
   * Fetch a key from the i18n file
   * @return the translation for key
   */
  public static String get(String key) {
    return get(key,null);
  }
  
  /**
   * Fetch a key from the i18n file and replace its variables with values
   * @param args the replacement values of the variables in the string
   */
  public static String get(String key, Object[] args) {
    String val=null;
    try {
      val = trans.get(key).toString();
    }
    catch (Exception e) {
      System.err.println("Missing translation: "+key);
      val="";
    }
    
    if (args==null) return val;
    else {
      MessageFormat formatter = new MessageFormat(val);
      return formatter.format(args);
    }
  }
  
  /**
   * Fetch a key from the i18n file and replace its variable with a value
   * @param args the replacement value of the variable in the string
   */
  public static String get(String key, Object arg) {
    Object[] tmp = {arg};
    return get(key,tmp);
  }
  
  private static boolean blob=false;
  
  /**
   * Called by the Toolloader upon loading a tool to add the tools i18n 
   * resource.
   * @param loader a classloader, through which rsrcname can be loaded.
   */
  public static void addBundle(ClassLoader loader) {
    try {
      
      ResourceBundle rb;
      
      /*
       * Work around a quirk in how resourcebundles are loaded. Problem is
       * that the file is loaded via ClassLoader.getResource(), which will
       * first delegate to it's parent and only if that fails tries itself.
       * For this reason the i18n file must have a different name in the
       * main program then in the jars and a test is required, which 
       * classloader is used for loading
       */
      if (loader==ClassLoader.getSystemClassLoader()) {
        rb  = ResourceBundle.getBundle(RSRCNAME);
      }
      else {
        rb  = ResourceBundle.getBundle(TOOLRSRCNAME, Locale.getDefault(),loader);
      }
      
      Enumeration e = rb.getKeys();
      while (e.hasMoreElements()) {
        String key = e.nextElement().toString();
        String val = rb.getString(key);
        trans.put(key,val);
        //System.err.println(loader+ " -> "+key+ " -> "+val );
      }
    }
    catch (Exception exp) {
      exp.printStackTrace();
    }
  }
  
}